#!/bin/sh
# the next line restarts using wish
# exec wish8.0 "$0" "$@"

# debug level

set debug(watch) 0
set debug(layout) 0
set debug(events) 0
set debug(info) 0

# we don't want to see any error messages if xpa is not available

set env(XPA_VERBOSITY) 0

# source other scripts
# we do it this way because mktclapp does not implement package require

source source.tcl

# required packages

package require tclxpa
package require Ast
package require iis
package require BLT
package require saotk

# let's start

set ds9(init) 1

# Set Application Name

if {"$argv0"=="ds9.tcl"} {
    set ds9(app) ds9
} else {
    set ds9(app) [file tail $argv0]
}

tk appname $ds9(app)
set ds9(title) "$ds9(app)"
set ds9(version) "1.9.7.2"

# First, look for help quest

ProcessHelp

# ds9 Variables
# General Note:
# ds9(frames) contains list of all frames
# ds9(active) contains list of active (show) frames
# active(<framename>) contains status show/hide for framename
# slave(<framename>) contains list of frames that are slaves to framename
# master(<framename>) contains the name of the master frame of framename

set ds9(top) .
set ds9(main) .ds9
set ds9(visual) ""
set ds9(depth) 8
set ds9(maxframes) 500
set ds9(frames) ""
set ds9(active) ""
set ds9(lock) 0
set ds9(next) ""
set ds9(mode) pointer
set ds9(dialog) motif
set ds9(scanOnLoad) 0
set ds9(cursor) 1
set ds9(xpa) 0
set ds9(displaymode) single

set ds9(b1) 0
set ds9(b2) 0
set ds9(b3) 0

set ds9(sb1) 0
set ds9(sb2) 0
set ds9(sb3) 0

set ds9(cb1) 0
set ds9(cb2) 0
set ds9(cb3) 0

set ds9(internalpad) 2
set ds9(framegap) 4

set ds9(freeze) 0

set ds9(lastFitsName) ""
set ds9(lastFitsSave) ""
set ds9(lastArrayName) ""
set ds9(lastURL) ""
set ds9(lastMosaicName) ""
set ds9(lastColormapName) ""
set ds9(lastMarkerName) "ds9.reg"
set ds9(lastTextName) "ds9.txt"
set ds9(lastTextSearch) ""
set ds9(lastTclName) ""
set ds9(lastGridName) "ds9.grid"
set ds9(lastContourName) ""
set ds9(lastLevelName) ""
set ds9(lastDSSName) "dss.fits"
set ds9(lastRASSName) "rass.fits"
set ds9(lastPixelName) "pixel.txt"

set ds9(fileProcs) {StdinFits ExternalFits CompressFits PackFits GzipFits}

set ds9(extFile) ".$ds9(app).file"
set ds9(extAnalysis) ".$ds9(app).analysis"
set ds9(analysisItem) 0

# About

set about(top) ".about"
set about(title) "About SAOImage DS9"
set about(message) "SAOImage DS9\nVersion $ds9(version)\n\nAuthors:\nWilliam Joye (SAO)\nEric Mandel (SAO)\nJohn Roll (SAO)\n\nContributions from:\nDong Mink (SAO) -wcssubs\nMark Calabretta (ATNF) -wcslib\nR.F. Warren-Smith (Starlink),\nD.S. Berry (Starlink) -AST\nP.T. Wallace (Starlink) -SLA\nDoug Tody (NOAO) -iis,zscale\nSteffen Traeger -ftp\nPeter Wilson (HEASARC),\nL. Brown (HEASARC) -contour\n"

# Help Variables

set help(top) .help
set help(mb) .helpMB
set help(try) 10
set help(ref) "http://hea-www.harvard.edu/RD/ds9/doc/ref/ref.html"
set help(user) "http://hea-www.harvard.edu/RD/ds9/doc/user/user.html"
set help(email) "saord@cfa.harvard.edu"

# Pixel Table Variables

set pixel(top) .pixel
set pixel(mb) .pixelMB
set pixel(view) 0
set pixel(size) 5
set pixel(maxsize) 9
set pixel(copy) {}

# Cursor Variables

set cursor(save) ""
set cursor(count) 0

# Crosshair Variables

set crosshair(lock) none

# Menu Variables

set menu(tearoff) 1
set menu(selectcolor) red

# Frame Variables

set current(frame) ""
set current(cursor) {}

set current(zoom) 1
set current(rotate) 0
set current(orient) none
set current(scale) linear

blt::vector histX histY

# Scale Variables

set scale(mode) minmax
set scale(scope) local
set minmax(mode) scan
set user(low) 0
set user(high) 100

# Marker Variables

set marker(shape) "circle"
set marker(color) green
set marker(edit) 1
set marker(move) 1
set marker(rotate) 1
set marker(delete) 1
set marker(include) 1
set marker(fixed) 0
set marker(maxdialog) 10
set marker(font) helvetica
set marker(fontSize) 10
set marker(fontStyle) normal
set marker(format) ds9
set marker(coord) physical
set marker(coordformat) degrees
set marker(delim) nl

# Default Marker Variables

set circle(radius) 20
set annulus(inner) 10
set annulus(outer) 30
set annulus(annuli) 1
set ellipse(radius1) 40
set ellipse(radius2) 20
set ellipseannulus(radius1) 40
set ellipseannulus(radius2) 20
set ellipseannulus(radius3) 60
set ellipseannulus(annuli) 1
set boxannulus(size1) 40
set boxannulus(size2) 20
set boxannulus(size3) 60
set boxannulus(annuli) 1
set box(width) 80
set box(height) 40

# WCS Variables

set wcs(align) 0
set wcs(system) fk5
set wcs(format) sexagesimal

# Colorbar Variables

set colorbar(name) ""
set colorbar(view) 1
set colorbar(width) 512
set colorbar(height) 12

set colorbar(map) Grey
set colorbar(invert) 0
set colorbar(count) 1
set colorbar(lock) 0
set colorbar(private) false

set colorbar(speed) 512
set colorbar(mult) 10
set colorbar(top) .clrbar
set colorbar(area) 0
set colorbar(mode) center

# Info Panel Variables

set info(view) 1
set info(name) ""
set info(filename) 1
set info(object) 0
set info(minmax) 0
set info(wcs) 1
set info(detector) 0
set info(amplifier) 0
set info(physical) 1
set info(image) 1
set info(frame) 1

# Display Coords Variables

set coord(fk4) 0
set coord(fk5) 1
set coord(icrs) 0
set coord(galactic) 0
set coord(ecliptic) 0
set coord(linear) 0
set coord(detector) 0
set coord(amplifier) 0
set coord(physical) 0
set coord(image) 1
set coord(format) sexagesimal
set coord(value) 1

# Buttons Variables

set buttons(name) ""
set buttons(view) 1

# Canvas Variables

set canvas(name) ""
set canvas(width) 512
set canvas(height) 512
set canvas(auto) 1
set canvas(size) 512

# Panner Variables

set pan(name) ""
set pan(view) 1
set pan(size) 128

set pan(imagecompass) 1
set pan(wcscompass) 1
set pan(wcstanpoint) 0
set pan(mode) click

# Magnifier Variables

set mag(name) ""
set mag(view) 1
set mag(size) 128
set mag(mode) accurate
set mag(region) 1
set mag(cursor) 1
set mag(zoom) 4

# Graph Variables

set graph(size) 150
set graph(font) "*helvetica-medium-r-*-100-*"
set graph(marginX) 35
set graph(marginY) 25
set graph(xMin) 0
set graph(xMax) 10
set graph(yMin) 1
set graph(yMax) 100
set graph(spacer) ""

set graphHorz(name) ""
set graphHorz(view) 0
set graphHorz(grid) 1
set graphHorz(log) false
blt::vector graphHorzX graphHorzY

set graphVert(name) ""
set graphVert(view) 0
set graphVert(grid) 1
set graphVert(log) false
blt::vector graphVertX graphVertY

# Printing Variables

set ps(dest) printer
set ps(cmd) "lp"
set ps(fileName) "ds9.ps"
set ps(orient) portrait
set ps(color) rgb
set ps(scale) scaled
set ps(size) letter
set ps(level) 2
set ps(interpolate) 0
set ps(resolution) 72
set ps(width) ""
set ps(height) ""

# Tile Variables

set tile(mode) grid

# Blink Variables

set blink(interval) 1000
set blink(index) -1
set blink(id) 0

# FITS Bin Table Support

set bin(function) sum
set bin(factor) 1
set bin(buffersize) 1024
set bin(help) "\t\[about <x> <y>\]\n\t\t\[buffersize <value>\]\n\t\t\[cols <x> <y>\]\n\t\t\[factor <value>\]\n\t\t\[filter <string>\]\n\t\t\[function average|sum\]\n\t\t\[to fit\]" \

set bindialog(top) .bl
set bindialog(mb) .blMB

# PanZoom Variables

set panZoom(top) .pz
set panZoom(mb) .pzMB
set panZoom(system) image
set panZoom(format) sexagesimal
set panZoom(speed) 512

# Contour Variables

set contour(top) .ct
set contour(mb) .ctMB
set contour(view) 0
set contour(color) green
set contour(smooth) 4
set contour(levels) 5
set contour(scale) linear
set contour(method) average
set contour(low) ""
set contour(high) ""
set contour(copy) ""

# Coordinate Grid

set grid(view) 0
set grid(top) .grid
set grid(mb) .gridMB

set grid(system) fk5
set grid(format) sexagesimal

set grid(type) int

set grid(grid) 1
set grid(gridcolor) 4
set grid(gridwidth) 1
set grid(gridstyle) 0

set grid(axes) 1
set grid(axescolor) 2
set grid(axeswidth) 1
set grid(axesstyle) 0

set grid(numlab) 1
set grid(numlabfont) helvetica
set grid(numlabstyle) bold
set grid(numlabsize) 10
set grid(numlabcolor) 3
set grid(numlabgap) .01

set grid(title) 1
set grid(titletext) ""
set grid(titledef) 1
set grid(titlefont) helvetica
set grid(titlestyle) bold
set grid(titlesize) 12
set grid(titlecolor) 0

set grid(textlab) 1
set grid(label1) ""
set grid(textlabdef1) 1
set grid(gap1) ""
set grid(gapunit1) ""
set grid(label2) ""
set grid(textlabdef2) 1
set grid(gap2) ""
set grid(gapunit2) ""
set grid(textlabfont) helvetica
set grid(textlabstyle) bold
set grid(textlabsize) 10
set grid(textlabcolor) 0
set grid(textlabgap1) .01
set grid(textlabgap2) .01

set grid(tickcolor) 1
set grid(tickwidth) 1
set grid(tickstyle) 0

set grid(border) 1
set grid(bordercolor) 3
set grid(borderwidth) 1
set grid(borderstyle) 0

set grid(labelling) interior
set grid(labelup) 0

# Examine Variables

set examine(mode) new
set examine(zoom) 4

# IIS Variables

set iis(state) 0
set iis(cursor) 0
set iis(abort) 0
set iis(ififo) none
set iis(ofifo) none
set iis(port) 5137
set iis(unix) "/tmp/.IMT%d"
set iis(width) 512
set iis(height) 512

# Name Server Variables

set nsvr(top) .nsvr
set nsvr(mb) .nsvrMB
set nsvr(server) ned-sao
set nsvr(system) fk5
set nsvr(format) degrees
set nsvr(name) ""
set nsvr(x) ""
set nsvr(y) ""

# DSS Variables

set dss(top) .dss
set dss(mb) .dssMB
set dss(server) sao
set dss(system) fk5
set dss(format) arcmin
set dss(name) ""
set dss(x) ""
set dss(y) ""
set dss(width) 15
set dss(height) 15
set dss(survey) dss
set dss(frame) new
set dss(save) 0
set dss(updatesize) 1

# Rass Variables

set rass(top) .rass
set rass(mb) .rassMB
set rass(server) mpe
set rass(system) fk5
set rass(format) arcmin
set rass(survey) broad
set rass(name) ""
set rass(x) ""
set rass(y) ""
set rass(width) 5
set rass(height) 5
set rass(frame) new
set rass(save) 0
set rass(updatesize) 1

# HTTP Variables

set http(log) 0
set http(timeout) 30

# FTP Variables

set ftp(log) 0

# Preference Variables

set prefs(file) "~/.$ds9(app).prefs"
set prefs(binfunction) $bin(function)
set prefs(binfactor) $bin(factor)
set prefs(binbuffersize) $bin(buffersize)
set prefs(wcsalign) $wcs(align)
set prefs(zoom) $current(zoom)
set prefs(orient) $current(orient)
set prefs(rotate) $current(rotate)
set prefs(scale) $current(scale)
set prefs(scalemode) $scale(mode)
set prefs(colorbarmap) $colorbar(map)
set prefs(colorbarinvert) $colorbar(invert)

# Load any preferences here, before we do any real work

LoadPrefs

# Toplevel-- Start here.

# We want to withdraw the window til everything is ready to go
wm withdraw $ds9(top)

set t "SAOImage $ds9(title)"
wm title $ds9(top) "$t"
wm iconname $ds9(top) "$t"

# This solves this problem:
# 1. tk no longer changes the vert size, thus generating two update sequences
# wm geometry $ds9(top) 750x749
# wm minsize $ds9(top) 640 480

$ds9(top) configure -menu .menuBar

# we need to set certain variables for color before anything else

ProcessColorCommandLine

set ds9(visual) [winfo visual .]
set ds9(depth) [winfo depth .]
if {$ds9(depth)==15} {
    set ds9(depth) 16
}

switch $ds9(visual)$ds9(depth) {
    pseudocolor8 {}
    truecolor8 {}
    truecolor16 {}
    truecolor24 {}
    default {BadVisualError}
}

# create our main frame

frame $ds9(main)

# we want the canvas to always to grow to consume all available space

grid rowconfigure $ds9(main) 2 -weight 1
grid columnconfigure $ds9(main) 0 -weight 1
pack $ds9(main) -fill both -expand true

# Create Colorbar-- Create this first, so in case of a private colormap,
# gui colors will be allocated in the new colormap, not the default colormap

CreateColorbar

# Create Canvas

set canvas(name) [canvas $ds9(main).image -width $canvas(width) \
		      -height $canvas(height) -bd 2 -relief groove \
		      -highlightthickness 0 -insertofftime 0]

# Create other parts of the display

CreateMenuBar
CreateInfoPanel
CreateButtons
CreateGraphs

# Make sure that the wm knows when to swap in the colormap (if needed)

wm colormapwindows . "$canvas(name)"

# Initialize the display

InitColorbar
InitPanner

# Set our current state of things

CreateColorMenus
UpdateInfoPanel
ChangeMode
LayoutView

# We need this, cause we have SetWatchCursor turned off, hense no idletasks
# for some reason, CreateFrame needs to have an idletasks before 
# creating a new frame

update idletasks
CreateFrame

# ok, ready to show the window

wm deiconify $ds9(top)

# we do this now, so configure events doesn't run LayoutFrames

InitCanvas

# Init external File Formats
# we want this before processing the command line

InitExternalFile

# Init analysis file formats

InitAnalysisFile

# and process any command line items
# we want to see something before any fits files are loaded

ProcessCommandLine

# Initialize XPA
# we need to do this after parsing the command line to get the correct title

catch {InitXPA}

# Init IIS

IISInit

# Load other user tcl code

catch {source "~/.$ds9(app)"}

# ok, we're done

set ds9(init) 0
