#ifndef SELF_CONTAINED
#include "regions.h"
#endif

/* adding this to FilterOpen will cause this module to be loaded
   for dynamic linking */
static int imregno=0;
void initimregions(void)
{
  imregno++;
  return;
}

static void marky(GFilt g, int sno, int flag)
{
  int i;
  /* mark include shape */
  if( flag ){
    for(i=max(g->ymin,g->shapes[sno].ystart);
	i<=min(g->ymax,g->shapes[sno].ystop);
	i++)
      g->ybuf[i] = 1;
  }
  /* mark exclude shape */
  else{
    for(i=g->ymin; i<=g->shapes[sno].ystart-1; i++)
      g->ybuf[i] = 1;
    for(i=g->shapes[sno].ystop+1; i<=g->ymax; i++)
      g->ybuf[i] = 1;
  }
}

/* ***************************** shape support ***************************** */

static void quadeq(double a, double b, double c, 
		   double *x1, double *x2, int *nr, int *nc)
{
  double dis, q;
  if(a == 0.0){
    *nc = 0;
    if(b == 0.0){
      *nr = 0; *x1 = 0.0;
    }
    else{
      *nr = 1; *x1 = -c / b;
    }
    *x2 = *x1;
  }
  else{
    dis = b*b - 4.0 * a * c;
    if( dis > 0.0 ){
      *nr = 2; *nc = 0;
      dis = sqrt(dis);
      if( b < 0.0 ) dis = -dis;
      q = -0.5 * (b + dis);
      *x1 = q/a; *x2 = c/q;
      if(*x1 > *x2){
	q = *x1; *x1 = *x2; *x2 = q;
      }
    } 
    else if( dis == 0.0 ){
      *nr = 1; *nc = 0; *x1 = - 0.5 * b / a; *x2 = *x1;
    }
    else{
      *nr = 0; *nc = 2; *x1 = - 0.5 * b / a; *x2 = 0.5 * sqrt(-dis) / a;
    }
  }
}

static void rgs_mark(GFilt g, Scan *scanlist, int xval, int yval )
{
  Scan scanmark, mark;
  /* since yval is used as an index, make sure its within limits */
  if(yval < g->ymin) yval = g->ymin;
  if(yval > g->ymax) yval = g->ymax;
  mark = (Scan)malloc(sizeof(ScanRec));
  mark->x = xval;
  /* starts are installed at back of list for given x */
  if( !scanlist[yval] || ((scanlist[yval])->x > xval) ){
    mark->next = scanlist[yval];
    scanlist[yval] = mark;
  } else {
    scanmark = scanlist[yval];
    while( scanmark->next && ((scanmark->next)->x < xval) )
      scanmark = scanmark->next;
    mark->next = scanmark->next;
    scanmark->next = mark;
  }
}

static void rgs_segment(GFilt g, Scan *scanlist, int width, int height, 
			double x1, double y1, double x2, double y2)
{
  int ystart, ystop, yval, xval;
  double invslope, xoffset;

  ystart = PIXINCL(y1);
  if( ystart < 1 ) ystart = 1;
  /* note: PIXINCL(stop) is 1st pixel not counted */
  ystop = PIXINCL(y2) - 1;
  if( ystop > height ) ystop = height;
  /* ignore segment if there is no positive slope in integer coords */
  if( (ystart > ystop) || (ystop < 1) )
    return;
  /* use inverse slope (run/rise) to get x given y with a multiply */
  invslope = (x1 - x2) / (y1 - y2);
  xoffset = x1 + ((ystart - y1) * invslope);
  for(yval=ystart; yval<=ystop; yval++){
    xval = PIXINCL(xoffset);
    /* clip line to edges of image area (actually bend line) */
    if(xval < 1)     xval = 1;
    if(xval > width) xval = width + 1;
    rgs_mark(g, scanlist, xval, yval);
    xoffset = xoffset + invslope;
  }
}

static void _polygoni(GFilt g, int qt, int rno, int sno, int flag,
		      double *vx, double *vy, int count)
{
  int i, j;
  double xlo, xhi;
  double ylo, yhi;

  /* divide by block factor to get real endpoints */
  for(i=0; i<count; i++){
    vx[i] /= (double)g->block;
    vy[i] /= (double)g->block;
  }

  /* find the limits */
  xlo = vx[0];
  xhi = xlo;
  ylo = vy[0];
  yhi = ylo;
  for(i=0; i<count; i++){
    if(vx[i] > xhi) xhi = vx[i];
    if(vx[i] < xlo) xlo = vx[i];
    if(vy[i] > yhi) yhi = vy[i];
    if(vy[i] < ylo) ylo = vy[i];
  }
  if( qt && (sno > 1) ){
    g->shapes[sno].ystart = g->shapes[sno-1].ystart;
    g->shapes[sno].ystop = g->shapes[sno-1].ystop;
  }
  else{
    g->shapes[sno].ystart = max(g->ymin,PIXINCL(ylo));
    g->shapes[sno].ystop = min(g->ymax,PIXINCL(yhi) - 1);
  }
  g->shapes[sno].scanlist = (Scan *)calloc(g->ymax+1, sizeof(Scan));
  marky(g, sno, flag);

  /* mark all horizontal segment crossings */
  /* start with segment between last and first point */
  j = count-1;
  for(i=0; i<count; i++){
    /* make segments always run from lower y to higher y */
    if(vy[i] > vy[j]){
      rgs_segment(g, g->shapes[sno].scanlist, g->xmax, g->ymax,
		  vx[j], vy[j], vx[i], vy[i]);
    }
    else{
      rgs_segment(g, g->shapes[sno].scanlist, g->xmax, g->ymax,
		  vx[i], vy[i], vx[j], vy[j]);
    }
    j = i;
  }
}

static int corner_vertex(int index, int width, int height,
			 double *x, double *y)
{
  switch (index) {
  case 1:
    *x = 0.0;
    *y = height + 1;
    break;
  case 2:
    *x = 0.0;
    *y = 0.0;
    break;
  case 3:
    *x = width + 1;
    *y = 0.0;
    break;
  case 4:
    *x = width + 1;
    *y = height + 1;
  default:
    break;
  }
  index = index + 1;
  if(index > 4) index = 1;
  return(index);
}

static int pie_intercept(int width, int height, double xcen, double ycen,
			 double angle, double *xcept, double *ycept)
{
  double angl, slope;	/* l: angle and slope of ray */
  angl = angle;
  /* put angles in normal range */
  while (angl < 0.0)
    angl = angl + 360.0;
  while (angl >= 360.0)
    angl = angl - 360.0;
  /* check for a horizontal angle */
  if(fabs(angl - 90.0) < SMALL_NUMBER) {
    *xcept = 0.0;
    *ycept = ycen;
    return(2);
  }
  if(fabs(angl - 270.0) < SMALL_NUMBER) {
    *xcept = width + 1;
    *ycept = ycen;
    return(4);
  }
  /* convert to a Cartesian angle */
  angl = angl + 90.0;
  if(angl >= 360.0)
    angl = angl - 360.0;
  if(angl < 180.0) {
    *ycept = height + 1;
    /* rule out vertical line */
    if(fabs(angl - 90.0) < SMALL_NUMBER) {
      *xcept = xcen;
      return(1);
    }
  } else {
    *ycept = 0.0;
    /* rule out vertical line */
    if(fabs(angl - 270.0) < SMALL_NUMBER) {
      *xcept = xcen;
      return(3);
    }
  }
  /* convert to radians */
  angl = (angl / 180.0) * PI;
  /* calculate slope */
  slope = tan(angl);
  /* calculate intercept with designated y edge */
  *xcept = xcen + ((*ycept - ycen) / slope);
  if(*xcept < 0) {
    *ycept = (ycen - (xcen * slope));
    *xcept = 0.0;
    return(2);
  } else if(*xcept > (width + 1)) {
    *ycept = (ycen + ((width + 1 - xcen) * slope));
    *xcept = width + 1;
    return(4);
  } else {
    if(*ycept < height)
      return(3);
    else
      return(1);
  }
}

void _impiei(GFilt g, int qt, int rno, int sno, int flag, double x, double y,
	    double xcen, double ycen, double angle1, double angle2)
{
  int width, height;		/* l: image mask width and height */
  double sweep;			/* l: sweep between cut angles */
  double vx[7], vy[7];		/* l: arrays of vertices for polygon */
  int count;			/* l: number of polygon vertices */
  int intrcpt1, intrcpt2;	/* l: side intercepted by each cut */
  double x2, y2;		/* l: coordinates of second intercept */

  /* NB: do not use x and y variables, they have bogus values */

  /* divide by block factor to get "real" parameters */
  xcen /= (double)g->block;
  ycen /= (double)g->block;
  /* temps */
  width = g->xmax;
  height = g->ymax;
  /* start listing vertices of polygon */
  vx[0] = xcen;
  vy[0] = ycen;
  sweep = angle2 - angle1;
  /* if sweep is too small to be noticed, don't bother */
  if(fabs(sweep) < SMALL_NUMBER)
    return;
  if (sweep < 0.0) sweep = sweep + 360.0;
  intrcpt1 = pie_intercept(width, height, xcen, ycen, angle1,
			   &(vx[1]), &(vy[1]));
  intrcpt2 = pie_intercept(width, height, xcen, ycen, angle2,
			   &x2, &y2);
  count = 2;
  /* if angles intercept same side and slice is between them, no corners */
  /* else, mark corners until reaching side with second angle intercept */
  if((intrcpt1 != intrcpt2) || (sweep > 180.0)){
    do{
      intrcpt1 = corner_vertex(intrcpt1, width, height,  
			       &(vx[count]), &(vy[count]));
      count = count + 1;
    }while(intrcpt1 != intrcpt2);
  }
  vx[count] = x2;
  vy[count] = y2;
  count++;
  _polygoni(g, qt, rno, sno, flag, vx, vy, count);
}

/* ***************************** shapes ********************************** */

void imannulusi(GFilt g, int rno, int sno, int flag, double x, double y,
		double xcen, double ycen, double iradius, double oradius)
{
  int yy;
  double dval;
  double xoff, yoff;
  Scan *scanlist;

  /* NB: do not use x and y variables, they have bogus values */

  /* divide by block factor to get "real" parameters */
  xcen /= (double)g->block;
  ycen /= (double)g->block;
  iradius /= (double)g->block;
  oradius /= (double)g->block;
  /* set y limits */
  g->shapes[sno].ystart = max(g->ymin,PIXSTART(ycen - oradius));
  g->shapes[sno].ystop = min(g->ymax,PIXSTOP(ycen + oradius));
  g->shapes[sno].scanlist = (Scan *)calloc(g->ymax+1, sizeof(Scan));
  scanlist = g->shapes[sno].scanlist;
  marky(g, sno, flag);
  /* calculate start/stop values for each y line */
  for(yy=g->shapes[sno].ystart; yy<=g->shapes[sno].ystop; yy++){
    yoff = PIXCEN(yy) - ycen;
    if( (dval=(oradius * oradius) - (yoff * yoff))>=0.0 ){
      xoff = sqrt(dval);
      rgs_mark(g, scanlist, PIXSTART(xcen - xoff), yy);
      rgs_mark(g, scanlist, PIXSTOP(xcen + xoff), yy);
      if( (dval=(iradius * iradius) - (yoff * yoff))>0.0 ){
	xoff = sqrt(dval);
	rgs_mark(g, scanlist, PIXSTART(xcen - xoff), yy);
	rgs_mark(g, scanlist, PIXSTOP(xcen + xoff), yy);
      }
    }
  }
}

int imannulus(GFilt g, int rno, int sno, int flag, double x, double y,
	      double xcen, double ycen, double iradius, double oradius)
{
  Scan scan;
  if( g->evsect ){
    y = (int)(y - g->tlminy + g->tloff);
    if(y < g->ymin) return 0;
    if(y > g->ymax) return 0;
    x = (int)(x - g->tlminx + g->tloff);
  }
  scan = g->shapes[sno].scanlist[(int)y];
  if( (scan							&& 
       ((y >= g->shapes[sno].ystart) && (y <= g->shapes[sno].ystop))	&&
       (scan->next->next ?
	(((x >= scan->x) && (x <= scan->next->next->next->x))	&&
	 !((x >= scan->next->x) && (x <= scan->next->next->x))) :
	((x >= scan->x) && (x <= scan->next->x))))
      ==flag ){
    if( flag ) g->rid = rno;
    return 1;
  }
  else
    return 0;
}

void imboxi(GFilt g, int rno, int sno, int flag, double x, double y,
	    double xcen, double ycen, double xwidth, double yheight,
	    double angle)
{
  double angl;			 /* l: Cartesian angle in radians */
  double half_width, half_height;/* l: radii (1/2 width and height) */
  double cosangl, sinangl;	 /* l: sine, cosine of the Cartesian angle */
  double hw_cos, hw_sin;	 /* l: products of half_width with sin, cos */
  double hh_cos, hh_sin;	 /* l: products of half_height with sin, cos */
  double cornerx[4], cornery[4]; /* l: arrays of x and y coords of 4 corners */

  /* NB: do not use x and y variables, they have bogus values */

  /* divide by block factor to get "real" parameters */
  xcen /= (double)g->block;
  ycen /= (double)g->block;
  xwidth /= (double)g->block;
  yheight /= (double)g->block;
  /* convert to a Cartesian angle; save angle for use in multi or slices */
  angl = angle + 90.0;
  while (angl >= 360.0) angl = angl - 360.0;
  /* convert to radians */
  angl = (angl / 180.0) * PI;
  sinangl = sin (angl);
  cosangl = cos (angl);
  /* since we rotate by 90.0 degrees to get from astro angle to cartesian, */
  /* we also need to switch the width and height. we do this secretly so */
  /* that the display will turn out right, by doing it in the half terms */
  half_width = yheight / 2.0;
  half_height = xwidth / 2.0;
  hw_cos = half_width * cosangl;
  hw_sin = half_width * sinangl;
  hh_cos = half_height * cosangl;
  hh_sin = half_height * sinangl;
  cornerx[0] = xcen - hw_cos - hh_sin;
  cornery[0] = ycen - hw_sin + hh_cos;
  cornerx[1] = xcen + hw_cos - hh_sin;
  cornery[1] = ycen + hw_sin + hh_cos;
  cornerx[2] = xcen + hw_cos + hh_sin;
  cornery[2] = ycen + hw_sin - hh_cos;
  cornerx[3] = xcen - hw_cos + hh_sin;
  cornery[3] = ycen - hw_sin - hh_cos;
  _polygoni(g, 0, rno, sno, flag, cornerx, cornery, 4);
}

int imbox(GFilt g, int rno, int sno, int flag, double x, double y,
	  double xcen, double ycen, double xwidth, double yheight,
	  double angle)
{
  return impolygon(g, rno, sno, flag, x, y);
}

void imcirclei(GFilt g, int rno, int sno, int flag, double x, double y,
	       double xcen, double ycen, double radius)
{
  int yy;
  double dval;
  double xoff, yoff;
  Scan *scanlist;

  /* NB: do not use x and y variables, they have bogus values */

  /* divide by block factor to get "real" parameters */
  xcen /= (double)g->block;
  ycen /= (double)g->block;
  radius /= (double)g->block;
  /* set y limits */
  g->shapes[sno].ystart = max(g->ymin,PIXSTART(ycen - radius));
  g->shapes[sno].ystop = min(g->ymax,PIXSTOP(ycen + radius));
  g->shapes[sno].scanlist = (Scan *)calloc(g->ymax+1, sizeof(Scan));
  scanlist = g->shapes[sno].scanlist;
  marky(g, sno, flag);
  /* calculate start/stop values for each y line */
  for(yy=g->shapes[sno].ystart; yy<=g->shapes[sno].ystop; yy++){
    yoff = PIXCEN(yy) - ycen;
    if( (dval=(radius * radius) - (yoff * yoff))>=0.0 ){
      xoff = sqrt(dval);
      rgs_mark(g, scanlist, PIXSTART(xcen - xoff), yy);
      rgs_mark(g, scanlist, PIXSTOP(xcen + xoff), yy);
    }
  }
}

int imcircle(GFilt g, int rno, int sno, int flag, double x, double y,
	     double xcen, double ycen, double radius)
{
  Scan scan;
  if( g->evsect ){
    y = (int)(y - g->tlminy + g->tloff);
    if(y < g->ymin) return 0;
    if(y > g->ymax) return 0;
    x = (int)(x - g->tlminx + g->tloff);
  }
  scan = g->shapes[sno].scanlist[(int)y];
  if( (scan 							&& 
      ((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))	&&
      ((x >= scan->x) && (x <= (scan->next)->x))) == flag ){
    if( flag ) g->rid = rno;
    return 1;
  }
  else{
   return 0;
  }
}

void imellipsei(GFilt g, int rno, int sno, int flag, double x, double y,
		double xcen, double ycen, double xrad, double yrad,
		double angle)
{
  int yy;
  int nr, nc;
  double yhi;
  double yoff;
  double xboff, xfoff;
  double angl;
  double sinangl, cosangl;
  double cossq, sinsq, xradsq, yradsq;
  double a, b_partial, c_partial;
  double b, c;
  Scan *scanlist;

  /* NB: do not use x and y variables, they have bogus values */

  /* divide by block factor to get "real" parameters */
  xcen /= (double)g->block;
  ycen /= (double)g->block;
  xrad /= (double)g->block;
  yrad /= (double)g->block;
  /* set worst case limits (xrad axis parallel to vertical axis) */
  /* convert to a Cartesian angle; save "angle" for use by other routines */
  angl = angle + 90.0;
  while( angl >= 360.0 )
    angl = angl - 360.0;
  /* convert to radians */
  angl = (angl / 180.0) * PI;
  sinangl = sin(angl);
  cosangl = cos(angl);
  /* calculate approximate y limits */
  /* choose lesser of containing rotbox and circle */
  yhi = fabs(sinangl * yrad) + fabs(cosangl * xrad);
  yhi = min(yhi, max(yrad, xrad));
  g->shapes[sno].ystart = max(g->ymin,PIXSTART(ycen - yhi));
  g->shapes[sno].ystop = min(g->ymax,PIXSTOP (ycen + yhi));
  g->shapes[sno].scanlist = (Scan *)calloc(g->ymax+1, sizeof(Scan));
  scanlist = g->shapes[sno].scanlist;
  marky(g, sno, flag);
  /* prepare partials for quadratic equation solutions to coordinates */
  cossq = cosangl * cosangl;
  sinsq = sinangl * sinangl;
  /* because we rotate by 90.0 degrees to get from astro angle to */
  /* cartesian, we also need to switch the x and y axes. we do this */
  /* secretly so that the display will turn out right, by doing it in */
  /* the sq terms */
  xradsq = yrad * yrad;
  yradsq = xrad * xrad;
  /* fill in as much of a,b,c as we can */
  a = (cossq / xradsq) + (sinsq / yradsq);
  b_partial = (2.0 * sinangl) * ((cosangl / xradsq) - (cosangl / yradsq));
  c_partial = (sinsq / xradsq) + (cossq / yradsq);
  /* calculate start/stop values for each y line */
  for(yy=g->shapes[sno].ystart; yy<=g->shapes[sno].ystop; yy++){
    yoff = yy - ycen;
    b = b_partial * yoff;
    c = (c_partial * yoff * yoff) - 1.0;
    /* solve quadratic */
    quadeq (a, b, c, &xboff, &xfoff, &nr, &nc);
    /* if real roots */
    if( nr != 0 ) {
      /* translate x coordinates */
      rgs_mark(g, scanlist, PIXSTART(xcen + xboff), yy);
      rgs_mark(g, scanlist, PIXSTOP(xcen + xfoff), yy);
    }
  }
}

int imellipse(GFilt g, int rno, int sno, int flag, double x, double y,
	      double xcen, double ycen, double xrad, double yrad, double angle)
{
  Scan scan;
  if( g->evsect ){
    y = (int)(y - g->tlminy + g->tloff);
    if(y < g->ymin) return 0;
    if(y > g->ymax) return 0;
    x = (int)(x - g->tlminx + g->tloff);
  }
  scan = g->shapes[sno].scanlist[(int)y];
  if( (scan 							&& 
      ((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))	&&
      ((x >= scan->x) && (x <= (scan->next)->x))) == flag ){
    if( flag ) g->rid = rno;
    return 1;
  }
  else
    return 0;
}

void imfieldi(GFilt g, int rno, int sno, int flag, double x, double y)
{
  /* NB: do not use x and y variables, they have bogus values */

  g->shapes[sno].ystart = g->ymin;
  g->shapes[sno].ystop = g->ymax;
  marky(g, sno, flag);
}

int imfield(GFilt g, int rno, int sno, int flag, double x, double y)
{
  if( flag ){
    g->rid = rno;
    return 1;
  }
  else{
    return 0;
  }
}

void imlinei(GFilt g, int rno, int sno, int flag, double x, double y,
	     double x1, double y1, double x2, double y2)
{
  double vx[2];
  double vy[2];
  int xval, yval;
  double invslope, xoffset;

  /* NB: do not use x and y variables, they have bogus values */

  /* divide by block factor to get "real" parameters */
  x1 /= (double)g->block;
  y1 /= (double)g->block;
  x2 /= (double)g->block;
  y2 /= (double)g->block;
  /* order by increasing y */
  if( y1 < y2 ){
    vx[0] = x1;
    vy[0] = y1;
    vx[1] = x2;
    vy[1] = y2;
  }
  else{
    vx[0] = x2;
    vy[0] = y2;
    vx[1] = x1;
    vy[1] = y1;
  }
  /* set y limits */
  g->shapes[sno].ystart = PIXNUM(vy[0]);
  g->shapes[sno].ystop = PIXNUM(vy[1]);
  g->shapes[sno].scanlist = (Scan *)calloc(g->ymax+1, sizeof(Scan));
  marky(g, sno, flag);
  if( feq(vy[0],vy[1]) ){
    rgs_mark(g, g->shapes[sno].scanlist, min(vx[0],vx[1]), vy[0]);
    rgs_mark(g, g->shapes[sno].scanlist, max(vx[0],vx[1]), vy[0]);
  }
  else{
    /* mark all horizontal segment crossings */
    invslope = (vx[0] - vx[1]) / (vy[0] - vy[1]);
    xoffset = vx[0];
    for(yval=vy[0]; yval<=vy[1]; yval++){
      xval = xoffset;
      rgs_mark(g, g->shapes[sno].scanlist, xval, yval);
      xoffset = xoffset + invslope;
    }
  }
}

int imline(GFilt g, int rno, int sno, int flag, double x, double y,
	   double x1, double y1, double x2, double y2)
{
  Scan scan;
  if( g->evsect ){
    y = (int)(y - g->tlminy + g->tloff);
    if(y < g->ymin) return 0;
    if(y > g->ymax) return 0;
    x = (int)(x - g->tlminx + g->tloff);
  }
  scan = g->shapes[sno].scanlist[(int)y];
  if( (scan                                     &&
      ((x==(int)scan->x)) 			||
       (scan->next&&((x>=(int)scan->x)&&(x<=(int)scan->next->x))))==flag ) {
    if( flag ) g->rid = rno;
    return 1;
  }
  else
    return 0;
}

void impiei(GFilt g, int rno, int sno, int flag, double x, double y,
	    double xcen, double ycen, double angle1, double angle2)
{
  _impiei(g, 0, rno, sno, flag, x, y, xcen, ycen, angle1, angle2);
}

int impie(GFilt g, int rno, int sno, int flag, double x, double y,
	  double xcen, double ycen, double angle1, double angle2)
{
  return impolygon(g, rno, sno, flag, x, y);
}

void imqtpiei(GFilt g, int rno, int sno, int flag, double x, double y,
	      double xcen, double ycen, double angle1, double angle2)
{
  _impiei(g, 1, rno, sno, flag, x, y, xcen, ycen, angle1, angle2);
}

int imqtpie(GFilt g, int rno, int sno, int flag, double x, double y,
	  double xcen, double ycen, double angle1, double angle2)
{
  return impolygon(g, rno, sno, flag, x, y);
}

void impointi(GFilt g, int rno, int sno, int flag, double x, double y,
	      double xcen, double ycen)
{
  /* NB: do not use x and y variables, they have bogus values */

  /* divide by block factor to get "real" parameters */
  xcen /= (double)g->block;
  ycen /= (double)g->block;
  /* set y limits */
  g->shapes[sno].ystart = PIXNUM(ycen);
  g->shapes[sno].ystop = PIXNUM(ycen);
  g->shapes[sno].scanlist = (Scan *)calloc(g->ymax+1, sizeof(Scan));
  marky(g, sno, flag);
  rgs_mark(g, g->shapes[sno].scanlist, PIXNUM(xcen), PIXNUM(ycen));
}

int impoint(GFilt g, int rno, int sno, int flag, double x, double y,
	    double xcen, double ycen)
{
  Scan scan;
  if( g->evsect ){
    y = (int)(y - g->tlminy + g->tloff);
    if(y < g->ymin) return 0;
    if(y > g->ymax) return 0;
    x = (int)(x - g->tlminx + g->tloff);
  }
  scan = g->shapes[sno].scanlist[(int)y];
  if( (scan                                     &&
      (y == (int)g->shapes[sno].ystart)		&& 
      (x == (int)scan->x))==flag )		{
    if( flag ) g->rid = rno;
    return 1;
  }
  else
    return 0;
}

#ifdef __STDC__
void
impolygoni(GFilt g, int rno, int sno, int flag, double x, double y, ...)
{
  double *vx=NULL, *vy=NULL;
  int count, maxcount;
  va_list args;
  va_start(args, y);
#else
void impolygoni(va_alist) va_dcl
{
  GFilt g;
  int rno, sno, flag;
  double x, y;
  double *vx=NULL, *vy=NULL;
  int count, maxcount;
  va_list args;
  va_start(args);

  g  =  va_arg(args, GFilt);
  rno  = va_arg(args, int);
  sno  = va_arg(args, int);
  flag  = va_arg(args, int);
  x = va_arg(args, double);
  y = va_arg(args, double);
#endif
  /* NB: do not use x and y variables, they have bogus values */

  /* allocate space for x,y arguments */
  maxcount = MASKINC;
  vx = (double *)malloc(maxcount*sizeof(double));
  vy = (double *)malloc(maxcount*sizeof(double));
  /* look for x,y arguments */
  count = 0;
  while( 1 ){
    if( (count + 1) >= maxcount ){
      maxcount += MASKINC;
      vx = (double *)realloc(vx, maxcount*sizeof(double));
      vy = (double *)realloc(vy, maxcount*sizeof(double));
    }
    vx[count] = va_arg(args, double);
    vy[count] = va_arg(args, double);
    if( feq(vx[count],PSTOP) && feq(vy[count],PSTOP) )
      break;
    count ++;
  }
  va_end(ap);
  /* realloc to actual size */
  vx = (double *)realloc(vx, count*sizeof(double));
  vy = (double *)realloc(vy, count*sizeof(double));
  /* call the common routine */
  _polygoni(g, 0, rno, sno, flag, vx, vy, count);
  if( vx ) free(vx);
  if( vy ) free(vy);
}

#ifdef __STDC__
int
impolygon(GFilt g, int rno, int sno, int flag, double x, double y, ...)
{
  int crossings;
  Scan scan;
  /*
  va_list args;
  va_start(args, y);
  */
#else
int impolygon(va_alist) va_dcl
{
  GFilt g;
  int rno, sno, flag;
  double x, y;
  int crossings;
  Scan scan;
  va_list args;
  va_start(args);
  g  = va_arg(args, GFilt);
  rno  = va_arg(args, int);
  sno  = va_arg(args, int);
  flag  = va_arg(args, int);
  x = va_arg(args, double);
  y = va_arg(args, double);
#endif
  if( g->evsect ){
    y = (int)(y - g->tlminy + g->tloff);
    if(y < g->ymin) return 0;
    if(y > g->ymax) return 0;
    x = (int)(x - g->tlminx + g->tloff);
  }
  /* no initialization of x for this row, just jump right in */
  if( (y>=g->shapes[sno].ystart)&&(y<=g->shapes[sno].ystop) ){
    crossings = 0;
    for(scan=g->shapes[sno].scanlist[(int)y]; scan; scan=scan->next){
      if( x >= scan->x )
	crossings++;
      else
	break;
    }
#ifdef SLOW
    if( fmod((double)crossings,2.0) == flag ){
#else
    if( (crossings%2) == flag ){
#endif
      if( flag ) g->rid = rno;
      return 1;
    }
    else
      return 0;
  }
  else
    return !flag;
}

