#include <strstream.h>

#include "basecircle.h"
#include "frame.h"
#include "util.h"

BaseCircle::BaseCircle(Frame* p, const Vector& ctr, double r,
		       const char* clr, const char* f, 
		       const char* t, unsigned short prop)
  : BaseMarker(p, ctr, 0, clr, f, t, prop)
{
  radii = NULL;
}

BaseCircle::~BaseCircle()
{
  if (radii)
    delete [] radii;
}

void BaseCircle::updateCoords(Matrix& m)
{
  for (int i=0; i<annuli; i++)
    radii[i] *= m[0][0];

  Marker::updateCoords(m);
}

void BaseCircle::draw()
{
  Marker::draw();
  includeExclude();
  render(parent->pixmap, parent->refToWidget, gc, parent->getZoom());
}

void BaseCircle::drawXOR()
{
  render(Tk_WindowId(parent->tkwin), parent->refToWindow, 
	 gcxor, parent->getZoom());
}

void BaseCircle::drawMagnifier(const Matrix& m)
{
  XSetForeground(display, gc, color);
  includeExclude();
  render(parent->magnifierPixmap, m, gc, parent->getMagnifierZoom());
}

void BaseCircle::ps(int mode)
{
  Marker::ps(mode);
  psIncludeExclude();

  Vector c = center*parent->refToCanvas;

  char buf[256];
  for (int i=0; i<annuli; i++) {
    double r = radii[i];
    if (!(properties & FIXED))
      r *= parent->getZoom();

    ostrstream str(buf,256);
    str << "newpath " << c.TkCanvasPs(parent->canvas) << r
	<< " 0 360 arc stroke" << endl << ends;
    Tcl_AppendResult(parent->interp, buf, NULL);
  }
}

int BaseCircle::isIn(const Vector& v)
{
  // v is in canvas coords

  double r = radii[annuli-1];
  if (properties & FIXED)
    r /= parent->getZoom();

  Vector p = v * parent->canvasToRef * Translate(-center);

  if ((p[0]*p[0] + p[1]*p[1])/(r*r) <= 1)
    return 1;
  else
    return 0;
}

// private

void BaseCircle::render(Drawable drawable, const Matrix& matrix, 
		    GC ggc, double zoom)
{
  for (int i=0; i<annuli; i++) {
    Vector c = center * matrix;
    Vector r = Vector(radii[i],radii[i]);

    if (!(properties & FIXED))
      r *= zoom;

    Vector ll = (c-r).round();
    Vector size = (r*2).round();

    // Verify size is positive
    // XDrawArc is sensative to bad data, and may hang the XServer

    if (size[0]>0 && size[1]>0)
      XDrawArc(display, drawable, ggc, (int)ll[0], (int)ll[1],
	       (int)size[0], (int)size[1], 0, 360*64);
  }
}
