#include <stdlib.h>
#include <string.h>

#include "colorscaletrue32.h"
#include "util.h"

ColorScaleTrueColor32::ColorScaleTrueColor32(int s, Visual* v, int msb) 
  : ColorScale(s)
{
  // this makes the assumption of one byte per color.

  colors_ = new unsigned char[s*4];
  int rs = decodeMask((unsigned long)v->red_mask);
  int gs = decodeMask((unsigned long)v->green_mask);
  int bs = decodeMask((unsigned long)v->blue_mask);
      
  // we need to check to byteswap when we have cross platforms

  if ((!msb && lsb()) || (msb && !lsb())) {
    for (int i=0; i<s; i++) {
      unsigned long r = psColors_[i*3+2];
      unsigned long g = psColors_[i*3+1];
      unsigned long b = psColors_[i*3];
      unsigned long a = 0;
      a |= r << rs;
      a |= g << gs;
      a |= b << bs;
      memcpy(colors_+i*4, &a, 4);
    }
  }
  else {
    for (int i=0; i<s; i++) {
      unsigned long r = psColors_[i*3+2];
      unsigned long g = psColors_[i*3+1];
      unsigned long b = psColors_[i*3];
      unsigned long a = 0;
      a |= r << rs;
      a |= g << gs;
      a |= b << bs;

      unsigned char* rr = (unsigned char*)(&a);
      *(colors_+i*4) = *(rr+3);
      *(colors_+i*4+1) = *(rr+2);
      *(colors_+i*4+2) = *(rr+1);
      *(colors_+i*4+3) = *(rr);
    }
  }
}

ColorScaleTrueColor32::~ColorScaleTrueColor32()
{
  if (colors_)
    delete [] colors_;
}

LinearScaleTrueColor32::LinearScaleTrueColor32(int s, 
					       unsigned short* indexCells,
					       unsigned char* colorCells,
					       int count, Visual* v, int msb) 
  : LinearScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor32(s, v, msb),
    ColorScale(s) {}

LogScaleTrueColor32::LogScaleTrueColor32(int s,
					 unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count, Visual* v, int msb)
  : LogScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor32(s, v, msb), 
    ColorScale(s) {}

SquaredScaleTrueColor32::SquaredScaleTrueColor32(int s,
						 unsigned short* indexCells,
						 unsigned char* colorCells,
						 int count, Visual* v, int msb)
  : SquaredScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor32(s, v, msb), 
    ColorScale(s) {}

SqrtScaleTrueColor32::SqrtScaleTrueColor32(int s,
					   unsigned short* indexCells,
					   unsigned char* colorCells,
					   int count, Visual* v, int msb)
  : SqrtScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor32(s, v, msb), 
    ColorScale(s) {}

IISScaleTrueColor32::IISScaleTrueColor32(unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count, Visual* v, int msb) 
  : IISScale(indexCells, colorCells, count), 
    ColorScaleTrueColor32(IISSIZE, v, msb),
    ColorScale(IISSIZE) {}
