/*
 *      Copyrights:
 *
 *    	  Copyright (c) 1996, 1998 Smithsonian Astrophysical Observatory
 *
 *        Permission to use, copy, modify, distribute, and  sell  this
 *        software  and  its  documentation  for any purpose is hereby
 *        granted without  fee,  provided  that  the  above  copyright
 *        notice  appear  in  all  copies and that both that copyright
 *        notice and this permission notice appear in supporting docu-
 *        mentation,  and  that  the  name  of the  Smithsonian Astro-
 *	  physical Observatory not be used in advertising or publicity
 *  	  pertaining to distribution of the software without specific,
 *	  written  prior  permission.   The Smithsonian  Astrophysical 
 *        Observatory makes no representations about  the  suitability
 *	  of  this  software for any purpose.  It is provided  "as is"
 *	  without express or implied warranty.
 *        THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  DISCLAIMS  ALL
 *	  WARRANTIES  WITH  REGARD  TO  THIS  SOFTWARE,  INCLUDING ALL
 *	  IMPLIED  WARRANTIES  OF  MERCHANTABILITY AND FITNESS, IN  NO
 *	  EVENT SHALL THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  BE 
 *	  LIABLE FOR  ANY SPECIAL, INDIRECT  OR  CONSEQUENTIAL DAMAGES
 *	  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS  OF USE,  DATA
 *	  OR  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *	  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 *	  THE  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *
 * acl.c -- xpa access control list management
 *
 */

#include <xpap.h>

/*
 *----------------------------------------------------------------------------
 *
 *
 *			Private Routines
 *
 *
 *----------------------------------------------------------------------------
 */

/* this is the head of the global list -- too lazy to do anything more */
static ACL aclhead=NULL;

#ifdef ANSI_FUNC
static ACL 
AclLookup (char *xclass, char *name, int ip)
#else
static ACL AclLookup(xclass, name, ip)
     char *xclass;
     char *name;
     int ip;
#endif
{
  ACL cur;
  /*  look for exact match */
  for(cur=aclhead; cur!=NULL; cur=cur->next){
    if( !strcmp(xclass, cur->xclass) 		&&
	!strcmp(name, cur->name)   		&&
	((cur->ip == -1) || (cur->ip == ip)) 	){
      return(cur);
    }
  }
  /* otherwise look for a template match */
  for(cur=aclhead; cur!=NULL; cur=cur->next){
    if( tmatch(xclass, cur->xclass)		&&
	tmatch(name, cur->name) 		&&
	((cur->ip == -1) || (cur->ip == ip)) 	){
      return(cur);
    }
  }
  return(NULL);
}

/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPAAclParse
 *
 * Purpose:	parse acl list into components
 *
 * Returns:	0 on success, -1 on failure
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
static int 
XPAAclParse (char *lbuf, char *xclass, char *name, int *ip, char *acl, int len)
#else
static int XPAAclParse(lbuf, xclass, name, ip, acl, len)
     char *lbuf;
     char *xclass;
     char *name;
     int *ip;
     char *acl;
     int len;
#endif
{
  char tbuf[SZ_LINE];
  int lp=0;

  /* class:name is required */
  if( word(lbuf, tbuf, &lp) ){
    XPAParseName(tbuf, xclass, name, len);
  }
  else
    return(-1);

  /* host is required but can be "*" for "all hosts" */
  if( word(lbuf, tbuf, &lp) ){
    if( !strcmp(tbuf, "*") )
      *ip = -1;
    else
      *ip = gethostip(tbuf);
  }
  else
    return(-1);

  /* acl is required */
  if( word(lbuf, tbuf, &lp) ){
    if( !strcmp(tbuf, "+") )
      strcpy(acl, XPA_ACLS);
    else if( !strcmp(tbuf, "-") )
      *acl = '\0';
    else
      strcpy(acl, tbuf);
    return(0);
  }
  else
    return(-1);
}

/*
 *----------------------------------------------------------------------------
 *
 *
 *		Semi-Public Routines (used by command.c)
 *
 *
 *----------------------------------------------------------------------------
 */

/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPAReceiveAcl
 *
 * Purpose:	add or modify the acl for this access point
 *
 * Returns:	0 for success, -1 for failure
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
int 
XPAReceiveAcl (void *client_data, void *call_data, char *paramlist,
	       char *buf, int len)
#else
int XPAReceiveAcl(client_data, call_data, paramlist, buf, len)
     void *client_data;
     void *call_data;
     char *paramlist;
     char *buf;
     int len;
#endif
{
  XPA xpa = (XPA)call_data;
  XPAComm comm;
  int i;
  char *s=NULL;
  char lbuf[SZ_LINE];
  char tbuf[SZ_LINE];

  if( paramlist && *paramlist ){
    s = paramlist;
    while( isspace(*s) )
      s++;
    sprintf(tbuf, "%s:%s %s\n", xpa->xclass, xpa->name, s);
    XPAAclEdit(tbuf);
  }
  else{
    while(XPARdl(xpa_datafd(xpa), lbuf, SZ_LINE, XPAShortTimeout()) ){
      sprintf(tbuf, "%s:%s %s\n", xpa->xclass, xpa->name, lbuf);
      XPAAclEdit(tbuf);
    }
  }

  /* reset all acl flags for this xpa so acl is recalculated */
  for(comm=xpa->commhead; comm!=NULL; comm=comm->next){
    for(i=0; i<4; i++){
      comm->acl[i] = -1;
    }
  }

  return(0);
}

/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPASendAcl
 *
 * Purpose:	return the acl for this access point
 *
 * Returns:	0 for success, -1 for failure
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
int 
XPASendAcl (void *client_data, void *call_data, char *paramlist,
	    char **buf, int *len)
#else
int XPASendAcl(client_data, call_data, paramlist, buf, len)
     void *client_data;
     void *call_data;
     char *paramlist;
     char **buf;
     int *len;
#endif
{
  XPA xpa = (XPA)call_data;
  ACL cur;
  int got = 0;
  char tbuf[SZ_LINE];

  /* zero all flags */
  for(cur=aclhead; cur!=NULL; cur=cur->next){
    cur->flag = 0;
  }
  /* look for exact matches */
  for(cur=aclhead; cur!=NULL; cur=cur->next){
    if(!strcmp(xpa->xclass, cur->xclass) && !strcmp(xpa->name, cur->name)){
      sprintf(tbuf, "%s:%s %s %s\n",
	      cur->xclass, cur->name, getiphost(cur->ip), cur->acl);
      send(xpa_datafd(xpa), tbuf, strlen(tbuf), 0);
      cur->flag = 1;
      got++;
    }
  }
  /* look for template matches that we have not printed yet */
  for(cur=aclhead; cur!=NULL; cur=cur->next){
    if( cur->flag == 0 ){
      if(tmatch(xpa->xclass, cur->xclass) && tmatch(xpa->name, cur->name)){
	sprintf(tbuf, "%s:%s %s %s\n",
		cur->xclass, cur->name, getiphost(cur->ip), cur->acl);
	send(xpa_datafd(xpa), tbuf, strlen(tbuf), 0);
	got++;
      }
    }
  }
  /* zero all flags */
  for(cur=aclhead; cur!=NULL; cur=cur->next){
    cur->flag = 0;
  }
  if( got == 0 ){
    send(xpa_datafd(xpa), "\n", 1, 0);
  }
  return(0);
}

/*
 *----------------------------------------------------------------------------
 *
 *
 *			Public Routines
 *
 *
 *----------------------------------------------------------------------------
 */

/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPAAclEdit
 *
 * Purpose:	add or modify acl entry in the xpa acl list
 *
 * Returns: 	0 on success, -1 on failure
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
int 
XPAAclEdit (char *lbuf)
#else
int XPAAclEdit(lbuf)
     char *lbuf;
#endif
{
  ACL cur;
  char xclass[SZ_LINE];
  char name[SZ_LINE];
  char acl[SZ_LINE];
  int ip;

  if( XPAAclParse(lbuf, xclass, name, &ip, acl, SZ_LINE) < 0 )
    return(-1);

  cur = AclLookup(xclass, name, ip);
  if( cur == NULL )
    return(XPAAclAdd(lbuf));
  else{
    if( cur->acl )
      free(cur->acl);
    cur->acl = NewString(acl);
    return(0);
  }
}


/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPAAclAdd
 *
 * Purpose:	add one acl entry to the xpa acl list
 *
 * Returns: 	0 on success, -1 on failure
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
int 
XPAAclAdd (char *lbuf)
#else
int XPAAclAdd(lbuf)
     char *lbuf;
#endif
{
  ACL xnew;
  ACL cur;
  char xclass[SZ_LINE];
  char name[SZ_LINE];
  char acl[SZ_LINE];
  int ip;

  /* allocate acl struct */
  if( (xnew = (ACL)calloc(1, sizeof(struct aclrec))) == NULL )
    goto error;

  /* parse info from line buffer */
  if( XPAAclParse(lbuf, xclass, name, &ip, acl, SZ_LINE) < 0 )
    goto error;

  /* fill in the blanks */
  xnew->xclass = NewString(xclass);
  xnew->name = NewString(name);
  xnew->ip = ip;
  xnew->acl = NewString(acl);

  /* add this acl to end of list of acl's */
  if( aclhead == NULL ){
    aclhead = xnew;
  }
  else{
    for(cur=aclhead; cur->next!=NULL; cur=cur->next)
      ;
    cur->next = xnew;
  }
  return(0);

error:
  if( xnew )
    free(xnew);
  return(-1);
}

/*
 *---------------------------------------------------------------------------
 *
 * Routine: 	XPAAclDel
 *
 * Purpose: 	free up alloc'ed memory in the acl record structure
 *
 * Results:	0 on success, -1 for failure
 *
 *---------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
int 
XPAAclDel (ACL acl)
#else
int XPAAclDel(acl)
     ACL acl;
#endif
{
  ACL cur;

  if( acl == NULL )
    return(-1);

  /* remove from list of acl's */
  if( aclhead ){
    if( aclhead == acl ){
      aclhead = aclhead->next;
    }
    else{
      for(cur=aclhead; cur!=NULL; cur=cur->next){
	if( cur->next == acl ){
	  cur->next = (cur->next)->next;
	  break;
	}
      }
    }
  }

  /* free up string space */
  if( acl->xclass ) free(acl->xclass);
  if( acl->name )  free(acl->name);
  if( acl->acl )   free(acl->acl);

  /* free up record struct */
  free((char *)acl);
  return(0);
}

/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPAAclFree
 *
 * Purpose:	
 *
 * Results:	1 on success, 0 for failure
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
void 
XPAAclFree (void)
#else
void XPAAclFree()
#endif
{
  ACL cur;
  ACL saveacl;

  for(cur=aclhead; cur!=NULL; ){
    saveacl = cur->next;
    XPAAclDel(cur);
    cur = saveacl;
  }
}

/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPAAclNew
 *
 * Purpose:	read or re-read the acl list
 *
 * Results:	number of lines in list (including default)
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
int 
XPAAclNew (char *aname, int flag)
#else
int XPAAclNew(aname, flag)
     char *aname;
     int flag;
#endif
{
  int got=0;
  char lbuf[SZ_LINE];
  char hostname[SZ_LINE];
  char *obuf;
  char *aclname=NULL;
  char *aclpath=NULL;
  char *defacl=NULL;
  char *keywords[10];
  char *values[10];
  int nkeys;
  FILE *fp;

  /* get acl file name */
  if( aname && *aname )
    aclname = aname;
  else if( (aclname=(char *)getenv("XPA_ACLFILE")) == NULL )
    aclname = XPA_ACLFILE;

  /* get the default acl */
  if( (defacl=(char *)getenv("XPA_DEFACL")) == NULL )
    defacl = XPA_DEFACL;

  /* if there is an old list, free it */
  if( flag == 0 )
    XPAAclFree();

  /* macro-expand it to deal with the host name */
  gethost(hostname, SZ_LINE);
  nkeys = 0;
  keywords[0] = "host";  values[0] = hostname; nkeys++;

  /* open the acl file */
  if( (aclpath = (char *)Access(aclname, "r")) &&
      (fp=fopen(aclpath, "r")) != NULL ){
    while( fgets(lbuf, SZ_LINE, fp) ){
      if( *lbuf == '#' ){
	continue;
      }
      if( (obuf = macro(lbuf, keywords, values, nkeys, NULL, NULL)) != NULL ){
	if( XPAAclAdd(obuf) == 0 )
	  got++;
	free(obuf);
      }
    }
    fclose(fp);
    free(aclpath);
  }

  /* add default acl (it very likely was overridden in the file) */
  if( (obuf = macro(defacl, keywords, values, nkeys, NULL, NULL)) != NULL ){
    if( XPAAclAdd(obuf) == 0 )
      got++;
    free(obuf);
  }

  /* return the news */
  return(got);
}

/*
 *----------------------------------------------------------------------------
 *
 * Routine:	XPAAclCheck
 *
 * Purpose:	validate an acl for a given class, name, and host
 *
 * Results:	1 on success, 0 on failure
 *
 *----------------------------------------------------------------------------
 */
#ifdef ANSI_FUNC
int 
XPAAclCheck (XPA xpa, int ip, int acl)
#else
int XPAAclCheck(xpa, ip, acl)
     XPA xpa;
     int ip;
     int acl;
#endif
{
  ACL cur;

  cur = AclLookup(xpa->xclass, xpa->name, ip);
  if( cur == NULL ){
    return(0);
  }
  else if( cur->acl == NULL ){
    return(0);
  }
  else{
    return(((char *)strchr(cur->acl, acl) != NULL));
  }
}
